%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This m-file includes Matlab code for the ionosphere data experiments
% reported in article "Compact Modeling of Data Using Independent
% Variable Group Analysis" by Esa Alhoniemi, Antti Honkela, Krista
% Lagus, Jeremias Sepp, Paul Wagner, and Harri Valpola
%

addpath /share/ivga/releases/ivga-1.14.5/program/ % change your own IVGA path here

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% create input data
% 
% The file ionosphere.data can be obtained from the UCI Machine
% Learning Repository
% http://www.ics.uci.edu/~mlearn/MLRepository.html

Data       = load('ionosphere.data', ',');
Data(:, 2) = []; % remove constant variable #2 from the
                 % dataset (contains no information for the
                 % classification)

Features  = Data(:,1:33); % feature vectors
Classes   = Data(:, 34);  % class information for the feature vectors

[nsamples nfeatures] = size(Features);

validsize     = 50;     % size of validation set
testsize      = 51;     % size of testing set
K             = 1:2:15; % optimal value for is found in this set
r             = 100;    % how many times each experiment is repeated
nivgaruns     = 100;    % number of ivga runs in each variable
                        % selection experiment using ivga

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% kNN run using all the variables
%

e = 0;
t = cputime;

for i = 1:r
  [Train Train_classes Valid Valid_classes Test Test_classes] = ...
      divide_data(Features, Classes, validsize, testsize);
  k      = select_k(Train, Train_classes, Valid, Valid_classes, K);
  Result = kNN(Train, Train_classes, Test, k);
  e      = e + sum(Result ~= Test_classes);

  save_svm_data(Train, Test, Train_classes, Test_classes, 'all', i);
end

err_all = e/(testsize * r);
t_all   = (cputime - t)/r;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% IVGA

t = cputime;
e = 0;
for i = 1:r
  [Train Train_classes Valid Valid_classes Test Test_classes] = ...
	divide_data(Features, Classes, validsize, testsize);
  Vars = ivga_selection(Train, Train_classes, nivgaruns);

  err_min = realmax;
  k_opt   = 0;
  for j = 1:nfeatures
    Vars_part = Vars(1:j);
    [k,err]   = select_k(Train(:,Vars_part), Train_classes, Valid(:,Vars_part), Valid_classes, K);
    if (err < err_min)
      err_min = err;
      k_opt   = k;
      Train_save = Train(:,Vars_part);
      Train_classes_save = Train_classes;
      Test_save = Test(:,Vars_part);
      Test_classes_save = Test_classes;
    end
  end

  Result = kNN(Train_save, Train_classes_save, Test_save, k_opt);

  e = e + sum(Result ~= Test_classes_save);

  save_svm_data(Train_save, Test_save, Train_classes_save, ...
                Test_classes_save, 'ivga', i);
end

err_ivga = e/(testsize * r);
t_ivga   = (cputime - t)/r;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Mann-Whitney test (aka Wilcoxon rank sum test)

t = cputime;
e = 0;
for i = 1:r
  [Train Train_classes Valid Valid_classes Test Test_classes] = ...
	divide_data(Features, Classes, validsize, testsize);
  Vars = mw_selection(Train, Train_classes);

  err_min = realmax;
  k_opt   = 0;
  for j = 1:nfeatures
    Vars_part = Vars(1:j);
    [k, err]  = select_k(Train(:,Vars_part), Train_classes, Valid(:,Vars_part), Valid_classes, K);
    if (err < err_min)
      err_min = err;
      k_opt = k;
      Train_save = Train(:,Vars_part);
      Train_classes_save = Train_classes;
      Test_save = Test(:,Vars_part);
      Test_classes_save = Test_classes;
    end
  end

  Result = kNN(Train_save, Train_classes_save, Test_save, k_opt);

  e = e + sum(Result ~= Test_classes_save);

  save_svm_data(Train_save, Test_save, Train_classes_save, ...
                Test_classes_save, 'mw', i);
end

err_mw = e/(testsize * r);
t_mw   = (cputime - t)/r;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% sequential forward selection run
%

t = cputime;
e = 0;
for i = 1:r
  [Train Train_classes Valid Valid_classes Test Test_classes] = ...
	divide_data(Features, Classes, validsize, testsize);
  Vars   = sffs(Train, Train_classes, 10, 40, 40, K, 3600);
  k      = select_k(Train, Train_classes, Valid, Valid_classes, K);
  Result = kNN(Train, Train_classes, Test, k);
  e      = e + sum(Result ~= Test_classes);

  save_svm_data(Train(:,Vars), Test(:,Vars), ...
                Train_classes, Test_classes, 'sffs', i);
end

err_sffs = e/(testsize * r);
t_sffs   = (cputime - t)/r;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% IVGA without validation set

t = cputime;
Error = zeros(nfeatures, length(K));
for i = 1:r
  [Train Train_classes Valid Valid_classes Test Test_classes] = ...
	divide_data(Features, Classes, validsize, testsize);
  Vars = ivga_selection([Train; Valid], [Train_classes; Valid_classes], nivgaruns);
  for j = 1:nfeatures
    Vars_part = Vars(1:j);
    for k = 1:length(K)
      Result = kNN([Train(:,Vars_part); Valid(:,Vars_part)], [Train_classes; Valid_classes], Test(:,Vars_part), K(k));
      Error(j, k) = Error(j, k) + sum(Result ~= Test_classes);
    end
  end
end

Error_ivga_novalid = Error/(testsize * r);
t_ivga_novalid     = (cputime - t)/r;

